@@ -166,8 +166,8 @@ samples <- function (bp,  which = 1:bp$g, col = ez.col, pch = 16,
   {
     label.col <- rep(NA, n)
     for (j in 1:g)
-      if (!is.na(match(j, which))) label.col[bp$group.aes==bp$g.names[j]] <- col[which==j]
-#    label.col <- stats::na.omit(label.col)
+      if (!is.na(match(j, which))) label.col[bp$group.aes==bp$g.names[j]] <- col[which==j][1]
+  #label.col <- stats::na.omit(label.col)
   }
   else
   {
@@ -591,7 +591,8 @@ control.concentration.ellipse <- function (g, g.names, df, kappa, which,
 #' This function allows the user to format the aesthetics for the category level points (CLPs).
 #'
 #' @param bp an object of class \code{biplot}.
-#' @param which a vector containing the columns or variables for which the CLPs should be displayed, with default \code{ncol(Xcat)}.
+#' @param which a vector containing the columns or variables for which the CLPs should be displayed, 
+#'        with default \code{1:ncol(Xcat)}.
 #' @param col the colour(s) for the CLPs, with default \code{black}.
 #' @param cex the character expansion(s) for the CLPs, with default \code{0.6}.
 #'
@@ -656,6 +657,61 @@ CLPs <- function (bp,  which = 1:ncol(bp$Xcat), col = "black", cex = 0.6)
   bp
 }
 # ----------------------------------------------------------------------------------------------
+#' Format aesthetics for the category level regions
+#'
+#' @description
+#' This function allows the user to format the aesthetics for the category level points (CLRs).
+#'
+#' @param bp an object of class \code{biplot}.
+#' @param which the column name or number for which the CLRs should be displayed, with default \code{1}. Only
+#'              one variable can be selected at a time.
+#' @param col the colours for the CLRs, with default \code{colorRampPalette(c("black","white"))}.
+#'
+#' @return The object of class \code{biplot} will be appended with a list called \code{CLP.aes} containing the following elements  A list with the following components is available:
+#' \item{which}{the variable number for which the CLRs are displayed.}
+#' \item{col}{the colours of the CLRs.}
+#'
+#' @seealso \code{\link{biplot}}, \code{\link{PCO}}, \code{\link{AoD}}
+#' 
+#' @usage
+#' CLRs (bp,  which = 1, col = "black")
+#' @aliases CLRs
+#'
+#' @export
+#'
+#' @examples 
+#' mtdf <- as.data.frame(mtcars)
+#' mtdf$cyl <- factor(mtdf$cyl)
+#' mtdf$vs <- factor(mtdf$vs)
+#' mtdf$am <- factor(mtdf$am)
+#' mtdf$gear <- factor(mtdf$gear)
+#' mtdf$carb <- factor(mtdf$carb)
+#' #biplot(mtdf[,-11], scaled = TRUE) |> PCO(group.aes = mtdf[,11]) |> 
+#' #CLRs(which = 10, col = "coral") |> plot()
+#' 
+CLRs <- function (bp,  which = 1, col = "black")
+{
+  p2 <- ncol(bp$Xcat)
+  if (!is.null(bp$CLR.aes$col)) col <- bp$CLR.aes$col
+
+  if (!is.null(which))
+  {
+    if (!all(is.numeric(which))) which <- match(which, colnames(bp$Xcat), nomatch = 0)
+    which <- which[1]
+  }
+  
+  # Expand col to length p2
+  col.len <- length(col)
+  num.lev <- nlevels(bp$Xcat[,which])
+  if (col.len > 1) { col <- col[ifelse(1:num.lev%%col.len==0, col.len, 1:num.lev%%col.len)]
+                   }
+  else {  if (col.len==1) col <- colorRampPalette(c("white",col))(num.lev)
+          else col <- colorRampPalette(c("white","black"))(num.lev)
+  }
+  bp$CLR.aes = list(which = which, col = col)
+  bp
+}
+# ----------------------------------------------------------------------------------------------
 #' Format aesthetics for the supplementary (new) biplot samples
 #'
 #' @description
@@ -711,7 +767,6 @@ newsamples <- function (bp,  col = "darkorange1", pch = 1, cex = 1,
                         label.offset = 0.5, connected=FALSE, 
                         connect.col = "black", connect.lty = 1, connect.lwd = 1)
 { 
-  
   if(!is.null(label.name) | !is.null(label.col) | 
      any(label.side!="bottom") | any(label.offset !=0.5) | any(label.cex!=0.75))
     label<-TRUE
@@ -873,13 +928,14 @@ biplot.density.2D.control <- function(g, g.names, which, contours = F, h = NULL,
                                       tcl = -0.2, mgp = c(0, -0.25, 0), layout.heights = c(100, 10), legend.mar = c(2, 5, 0, 5)) 
 {
   if (!is.null(which))
-    if (which == "all") which <- 0
-  else if (!all(is.numeric(which))) which <- match(which, g.names, nomatch = 0)
+    if (!all(is.numeric(which))) which <- match(which, g.names, nomatch = 0)
   which <- which[which <= g]
   which <- which[which >= 0]
-  if (!is.null(which)) which <- which[1]
+  #if (!is.null(which)) which <- which[1]
   list(which = which, contours = contours, h = h, n = n, col = col, contour.col = contour.col,
        cuts = cuts, cex = cex, tcl = tcl, mgp = mgp, layout.heights = layout.heights, legend.mar = legend.mar)
+
+  
 }
 
 
